#!/bin/bash

# RHTS script for detection of avc denied services
# Testing approach is by recommendation of Bill Nottingham <notting@redhat.com>
#
# 2006/10/12 Josef Kubin <jkubin@redhat.com>

OUTPUTFILE="/tmp/RHTServices4AVCDeny.log.$(date +%s)"

exec 5>&1 6>&2
exec >> "${OUTPUTFILE}" 2>&1

# source the test script helpers
. /usr/bin/rhts_environment.sh

trap checklog USR1
trap runlevelstable USR2

RESULT='FAIL'

# counter - how many transition problems with SELinux policy were detected
SCORE=0

SEP='%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'

SEP2='~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'

LOGFILE='/var/log/audit/audit.log'

# sign for idle looop
LOOP=true;

# here is a list of services, which breaks testing or remote access, they won't be tested!!!
DONTEST='netplugd network single sshd auditd halt killall NetworkManager NetworkManagerDispatcher';

# timestamp from last line
LASTIMESTAMP=`sed -n '$s/^.*audit(\([^\.]*\).*/\1/p' $LOGFILE`

# idle loop
idle () {
	while $LOOP
	do
	    usleep 10000
	done

	LOOP=true
}

# interrupter for idle loop
runlevelstable () {
        LOOP=false;
}

# runlevel service transition handler for USR1 signal start/stop of servcie (I've modified /etc/rc.d/rc for it) 
checklog () {

	IFS=''

	# save the latest timestamp from the last line
	LATESTIMESTAMP=`sed -n '$s/^.*audit(\([^\.]*\).*/\1/p' $LOGFILE`

	# if the timestamps are different then check appended the new lines till end of log file for an "avc:  denied" substring
	if [ $LASTIMESTAMP != $LATESTIMESTAMP ]; then

		# It will print everything from $LASTIMESTAMP till end, on condition of occurence 'avc:  denied' substring somewhere
		# The line(s) with $LASTIMESTAMP is/are excluded from testing of course
		AVCDENIED=`sed -n '/'"$LASTIMESTAMP"'/!b;:y;n;/'"$LASTIMESTAMP"'/by;:x;$!{N;bx};/\<avc:  denied\>/p' $LOGFILE`

		if [[ $AVCDENIED ]]; then

			(( SCORE++ ))

			echo $SEP
			echo `date +%s` `cat /tmp/CurrentService` has been detected as avc denied
			echo Latest appended lines from $LOGFILE
			echo $SEP2
			echo $AVCDENIED
			echo $SEP

		else
			echo `date +%s` `cat /tmp/CurrentService` - some lines were appended to $LOGFILE
			#   echo The last line:
			#   tail -n1 $LOGFILE
		fi

		LASTIMESTAMP=$LATESTIMESTAMP

	else

		echo `date +%s` `cat /tmp/CurrentService`

	fi

	IFS=$'\n'

}

if [ ! -e /etc/init.d/rhtshelper ]; then
	{

	echo '#!/bin/bash'
	echo
	echo '# description:  This script is for testing support of all services for \ '
	echo '#               detection of problems with SELinux'
	echo
	echo 'cleaning () {'
	chkconfig --list|sed '/^$/q;s_^\([^\t ]*\)[^3]*3:\([^\t ]*\).*_chkconfig --level 3 \1 \2_;'
	echo
	echo 'chkconfig --level 1 sshd off'
	echo 'chkconfig --level 1 network off'
	echo 'chkconfig --level 1 auditd off'
	echo 'mv -f /etc/rc.d/{rc.backup,rc}'
	echo 'rm -f /tmp/CurrentService'
	echo '}'
	echo
	echo 'suicide () {'
	echo '      rm -f /etc/init.d/rhtshelper /etc/rc.d/rc1.d/S99rhtshelper /etc/rc.d/rc{2..5}.d/S99zrhtshelper'
	echo '}'
	echo
	echo 'case "$1" in'
	echo ' start)'
	echo '      killall -s USR2 runtest.sh'
	echo '      ;;'
	echo ' clean)'
	echo '      cleaning'
	echo '      suicide'
	echo '      ;;'
	echo '     *)'
	echo '     echo {start|clean}'
	echo 'esac'

	} > /etc/init.d/rhtshelper

	chmod u+x /etc/init.d/rhtshelper

	# create some necessary symlinks
	ln -s /etc/init.d/rhtshelper /etc/rc.d/rc1.d/S99rhtshelper
	ln -s /etc/init.d/rhtshelper /etc/rc.d/rc2.d/S99zrhtshelper
	ln -s /etc/init.d/rhtshelper /etc/rc.d/rc3.d/S99zrhtshelper
	ln -s /etc/init.d/rhtshelper /etc/rc.d/rc4.d/S99zrhtshelper
	ln -s /etc/init.d/rhtshelper /etc/rc.d/rc5.d/S99zrhtshelper

	chkconfig --level 1 sshd on
	chkconfig --level 1 network on
	chkconfig --level 1 auditd on

	# dirty trick for detection of start/stop service and transition lock between runlevels - this script needs listen signals SIGUSR1 SIGUSR2
	grep -q '/tmp/CurrentService' /etc/rc.d/rc || sed -i.backup '/^done/s__echo -n $i $? > /tmp/CurrentService\nkillall -s USR1 runtest.sh\nsleep 1\n&_' /etc/rc.d/rc
fi

################################################################################
### Main executive part ########################################################
################################################################################

echo $SEP2
echo '# rpm -q selinux-policy-targeted'

rpm -q selinux-policy-targeted

echo $SEP2
echo '# sestatus'
sestatus
echo $SEP2
echo Switching on of all services for runlevel 3 ...
echo

COUNT=0;

# this part switches on every service installed on tested system
for i in `chkconfig --list|sed '/^$/q;s_^\([^\t ]*\).*_\1_'`
do
  if [[ $DONTEST =~ "\<$i\>" ]]
  then
    echo Service \"$i\" hasn\'t been tested because it is dangerous - should be tested locally
  else
    #chkconfig --level 3 $i on
    (( COUNT++ ))
    echo $i on
  fi
done


echo $SEP2
echo $COUNT services will be tested.
echo Switching on of all services for runlevel 3 was finished.
echo $SEP2
echo

set `/sbin/runlevel`
if [ $2 != "1" -a $2 != "S" ]; then
  echo Starting transition into runlevel 1
  echo
sleep 3
  telinit 1
  idle
  echo
  echo Transition into runlevel 1 was finished
echo
fi

echo Starting transition into runlevel 3
echo
sleep 3
telinit 3
idle

echo
echo Transition into runlevel 3 was finished
echo
echo Starting transition into runlevel 1
echo
sleep 3
telinit 1
idle
echo
echo Transition into runlevel 1 was finished
echo
echo Cleaning garbage after testing ...

service rhtshelper clean

echo Finished.
echo
echo Starting ordinary transition into runlevel 3 ...
echo

telinit 3

echo All testing has been finished

[ $SCORE -eq 0 ] && RESULT='PASS'

exec 1>&5 2>&6

report_result $TEST $RESULT $SCORE

